<?php

/**
 * @file
 * Content moderation notifications token support.
 */

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Render\BubbleableMetadata;

/**
 * Implements hook_token_info().
 */
function content_moderation_notifications_token_info() {

  $type = [
    'name' => t('Content moderation states'),
    'description' => t('Content moderation state transition tokens.'),
    'needs-data' => 'entity',
  ];

  $tokens['workflow'] = [
    'name' => t('Workflow'),
    'description' => t('The name of the corresponding workflow.'),
  ];

  $tokens['from-state'] = [
    'name' => t('Old moderation state'),
    'description' => t('The previous state of the moderated content.'),
  ];

  $tokens['to-state'] = [
    'name' => t('Current moderation state'),
    'description' => t('The new/current state of the moderated content.'),
  ];

  return [
    'types' => ['content_moderation_notifications' => $type],
    'tokens' => [
      'content_moderation_notifications' => $tokens,
    ],
  ];
}

/**
 * Implements hook_tokens().
 */
function content_moderation_notifications_tokens($type, $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata) {

  $replacements = [];
  if ($type === 'content_moderation_notifications' && isset($data['entity']) && $data['entity'] instanceof ContentEntityInterface) {
    $entity = $data['entity'];

    /** @var \Drupal\content_moderation_notifications\NotificationInformationInterface $notification_information */
    $notification_information = \Drupal::service('content_moderation_notifications.notification_information');

    if ($notification_information->isModeratedEntity($entity)) {

      foreach ($tokens as $name => $original) {
        switch ($name) {
          case 'workflow':
            $workflow = $notification_information
              ->getWorkflow($entity)
              ->label();
            $replacements[$original] = $workflow;
            $bubbleable_metadata->addCacheableDependency($workflow);
            break;

          case 'from-state':
            if ($transition = $notification_information->getTransition($entity)) {
              $replacements[$original] = $notification_information->getPreviousState($entity)->label();
              $bubbleable_metadata->addCacheableDependency($transition);
            }
            break;

          case 'to-state':
            if ($transition = $notification_information->getTransition($entity)) {
              $replacements[$original] = $transition->to()->label();
              $bubbleable_metadata->addCacheableDependency($transition);
            }
            break;
        }
      }
    }
  }

  return $replacements;
}
